@@ -50,10 +50,10 @@ typedef enum {
   NDPI_PROTOCOL_POSTGRES              = 19,
   NDPI_PROTOCOL_MYSQL                 = 20,
   NDPI_PROTOCOL_MS_OUTLOOK            = 21, /* Hotmail / Microsoft Outlook / Exchange */
-  NDPI_PROTOCOL_DIRECT_DOWNLOAD_LINK  = 22,
+  NDPI_PROTOCOL_FREE_22               = 22, /* FREE */
   NDPI_PROTOCOL_MAIL_POPS             = 23,
-  NDPI_PROTOCOL_APPLEJUICE            = 24,
-  NDPI_PROTOCOL_DIRECTCONNECT         = 25,
+  NDPI_PROTOCOL_TAILSCALE             = 24,
+  NDPI_PROTOCOL_FREE_25               = 25, /* FREE */
   NDPI_PROTOCOL_NTOP                  = 26,
   NDPI_PROTOCOL_COAP                  = 27,
   NDPI_PROTOCOL_VMWARE                = 28,
@@ -61,8 +61,8 @@ typedef enum {
   NDPI_PROTOCOL_DTLS                  = 30,
   NDPI_PROTOCOL_UBNTAC2               = 31, /* Ubiquity UBNT AirControl = 2 */
   NDPI_PROTOCOL_KONTIKI               = 32,
-  NDPI_PROTOCOL_OPENFT                = 33,
-  NDPI_PROTOCOL_FASTTRACK             = 34,
+  NDPI_PROTOCOL_FREE_33               = 33, /* FREE */
+  NDPI_PROTOCOL_FREE_34               = 34, /* FREE */
   NDPI_PROTOCOL_GNUTELLA              = 35,
   NDPI_PROTOCOL_EDONKEY               = 36,
   NDPI_PROTOCOL_BITTORRENT            = 37,
@@ -84,17 +84,17 @@ typedef enum {
   NDPI_PROTOCOL_CPHA                  = 53,
   NDPI_PROTOCOL_PPSTREAM              = 54,
   NDPI_PROTOCOL_ZATTOO                = 55,
-  NDPI_PROTOCOL_SHOUTCAST             = 56,
-  NDPI_PROTOCOL_SOPCAST               = 57,
+  NDPI_PROTOCOL_FREE_56               = 56, /* FREE */
+  NDPI_PROTOCOL_FREE_57               = 57, /* FREE */
   NDPI_PROTOCOL_DISCORD               = 58,
   NDPI_PROTOCOL_TVUPLAYER             = 59,
   NDPI_PROTOCOL_MONGODB               = 60,
   NDPI_PROTOCOL_PLURALSIGHT           = 61,
-  NDPI_PROTOCOL_THUNDER               = 62,
+  NDPI_PROTOCOL_FREE_62               = 62, /* FREE */
   NDPI_PROTOCOL_OCSP                  = 63,
   NDPI_PROTOCOL_VXLAN                 = 64,
   NDPI_PROTOCOL_IRC                   = 65,
-  NDPI_PROTOCOL_AYIYA                 = 66,
+  NDPI_PROTOCOL_MERAKI_CLOUD          = 66,
   NDPI_PROTOCOL_JABBER                = 67,
   NDPI_PROTOCOL_NATS                  = 68,
   NDPI_PROTOCOL_AMONG_US              = 69,
@@ -126,8 +126,8 @@ typedef enum {
   NDPI_PROTOCOL_IAX                   = 95,
   NDPI_PROTOCOL_TFTP                  = 96,
   NDPI_PROTOCOL_AFP                   = 97,
-  NDPI_PROTOCOL_STEALTHNET            = 98,
-  NDPI_PROTOCOL_AIMINI                = 99,
+  NDPI_PROTOCOL_FREE_98               = 98, /* FREE */
+  NDPI_PROTOCOL_FREE_99               = 99, /* FREE */
   NDPI_PROTOCOL_SIP                   = 100,
   NDPI_PROTOCOL_TRUPHONE              = 101,
   NDPI_PROTOCOL_IP_ICMPV6             = 102,
@@ -135,8 +135,8 @@ typedef enum {
   NDPI_PROTOCOL_ARMAGETRON            = 104,
   NDPI_PROTOCOL_CROSSFIRE             = 105,
   NDPI_PROTOCOL_DOFUS                 = 106,
-  NDPI_PROTOCOL_FIESTA                = 107,
-  NDPI_PROTOCOL_FLORENSIA             = 108,
+  NDPI_PROTOCOL_FREE_107              = 107, /* FREE */
+  NDPI_PROTOCOL_FREE_108              = 108, /* FREE */
   NDPI_PROTOCOL_GUILDWARS             = 109,
   NDPI_PROTOCOL_AMAZON_ALEXA          = 110,
   NDPI_PROTOCOL_KERBEROS              = 111,
@@ -336,6 +336,32 @@ typedef enum {
   NDPI_PROTOCOL_THREEMA               = 305,
   NDPI_PROTOCOL_ALICLOUD              = 306,
   NDPI_PROTOCOL_AVAST                 = 307,
+  NDPI_PROTOCOL_TIVOCONNECT           = 308,
+  NDPI_PROTOCOL_KISMET                = 309,
+  NDPI_PROTOCOL_FASTCGI               = 310,
+  NDPI_PROTOCOL_FTPS                  = 311,
+  NDPI_PROTOCOL_NATPMP                = 312,
+  NDPI_PROTOCOL_SYNCTHING             = 313,
+  NDPI_PROTOCOL_CRYNET                = 314,
+  NDPI_PROTOCOL_LINE                  = 315,
+  NDPI_PROTOCOL_LINE_CALL             = 316,
+  NDPI_PROTOCOL_APPLETVPLUS           = 317,
+  NDPI_PROTOCOL_DIRECTV               = 318,
+  NDPI_PROTOCOL_HBO                   = 319,
+  NDPI_PROTOCOL_VUDU                  = 320,
+  NDPI_PROTOCOL_SHOWTIME              = 321,
+  NDPI_PROTOCOL_DAILYMOTION           = 322,
+  NDPI_PROTOCOL_LIVESTREAM            = 323,
+  NDPI_PROTOCOL_TENCENTVIDEO          = 324,
+  NDPI_PROTOCOL_IHEARTRADIO           = 325,
+  NDPI_PROTOCOL_TIDAL                 = 326,
+  NDPI_PROTOCOL_TUNEIN                = 327,
+  NDPI_PROTOCOL_SIRIUSXMRADIO         = 328,
+  NDPI_PROTOCOL_MUNIN                 = 329,
+  NDPI_PROTOCOL_ELASTICSEARCH         = 330,
+  NDPI_PROTOCOL_TUYA_LP               = 331, /* TUYA LAN Protocol; IoT OS: https://github.com/tuya/tuya-iotos-embeded-sdk-wifi-ble-bk7231n */
+  NDPI_PROTOCOL_TPLINK_SHP            = 332, /* TP-LINK Smart Home Protocol */
+
 
 #ifdef CUSTOM_NDPI_PROTOCOLS
 #include "../../../nDPI-custom/custom_ndpi_protocol_ids.h"
