@@ -28,8 +28,6 @@
 #include "ndpi_encryption.h"
 
 extern char *strptime(const char *s, const char *format, struct tm *tm);
-extern int processTLSBlock(struct ndpi_detection_module_struct *ndpi_struct,
-                           struct ndpi_flow_struct *flow);
 extern int processClientServerHello(struct ndpi_detection_module_struct *ndpi_struct,
 				    struct ndpi_flow_struct *flow, uint32_t quic_version);
 extern int http_process_user_agent(struct ndpi_detection_module_struct *ndpi_struct,
@@ -148,18 +146,18 @@ static u_int32_t __get_master(struct ndpi_detection_module_struct *ndpi_struct,
 
 /* **************************************** */
 
-void ndpi_search_tls_tcp_memory(struct ndpi_detection_module_struct *ndpi_struct,
-				struct ndpi_flow_struct *flow) {
-  struct ndpi_packet_struct *packet = &ndpi_struct->packet;
-  message_t *message = &flow->l4.tcp.tls.message[packet->packet_direction];
+static int ndpi_search_tls_memory(struct ndpi_detection_module_struct *ndpi_struct,
+				  struct ndpi_flow_struct *flow,
+				  const u_int8_t *payload,
+				  u_int16_t payload_len,
+				  u_int32_t seq,
+				  message_t *message) {
   u_int avail_bytes;
 
-  /* TCP */
 #ifdef DEBUG_TLS_MEMORY
-  printf("[TLS Mem] Handling TCP/TLS flow [payload_len: %u][buffer_len: %u][direction: %u]\n",
-	 packet->payload_packet_len,
-	 message->buffer_len,
-	 packet->packet_direction);
+  printf("[TLS Mem] Handling TLS flow [payload_len: %u][buffer_len: %u]\n",
+	 payload_len,
+	 message->buffer_len);
 #endif
 
   if(message->buffer == NULL) {
@@ -168,7 +166,7 @@ void ndpi_search_tls_tcp_memory(struct ndpi_detection_module_struct *ndpi_struct
     message->buffer = (u_int8_t*)ndpi_malloc(message->buffer_len);
 
     if(message->buffer == NULL)
-      return;
+      return -1;
 
 #ifdef DEBUG_TLS_MEMORY
     printf("[TLS Mem] Allocating %u buffer\n", message->buffer_len);
@@ -177,11 +175,11 @@ void ndpi_search_tls_tcp_memory(struct ndpi_detection_module_struct *ndpi_struct
 
   avail_bytes = message->buffer_len - message->buffer_used;
 
-  if(avail_bytes < packet->payload_packet_len) {
-    u_int new_len = message->buffer_len + packet->payload_packet_len - avail_bytes + 1;
+  if(avail_bytes < payload_len) {
+    u_int new_len = message->buffer_len + payload_len - avail_bytes + 1;
     void *newbuf  = ndpi_realloc(message->buffer,
 				 message->buffer_len, new_len);
-    if(!newbuf) return;
+    if(!newbuf) return -1;
 
 #ifdef DEBUG_TLS_MEMORY
     printf("[TLS Mem] Enlarging %u -> %u buffer\n", message->buffer_len, new_len);
@@ -192,39 +190,38 @@ void ndpi_search_tls_tcp_memory(struct ndpi_detection_module_struct *ndpi_struct
     avail_bytes = message->buffer_len - message->buffer_used;
   }
 
-  if(packet->payload_packet_len > 0 && avail_bytes >= packet->payload_packet_len) {
+  if(payload_len > 0 && avail_bytes >= payload_len) {
     u_int8_t ok = 0;
 
     if(message->next_seq != 0) {
-      if(ntohl(packet->tcp->seq) == message->next_seq)
+      if(seq == message->next_seq)
 	ok = 1;
     } else
       ok = 1;
 
     if(ok) {
       memcpy(&message->buffer[message->buffer_used],
-	     packet->payload, packet->payload_packet_len);
+	     payload, payload_len);
 
-      message->buffer_used += packet->payload_packet_len;
+      message->buffer_used += payload_len;
 #ifdef DEBUG_TLS_MEMORY
-      printf("[TLS Mem] Copied data to buffer [%u/%u bytes][direction: %u][tcp_seq: %u][next: %u]\n",
+      printf("[TLS Mem] Copied data to buffer [%u/%u bytes][tcp_seq: %u][next: %u]\n",
 	     message->buffer_used, message->buffer_len,
-	     packet->packet_direction,
-	     ntohl(packet->tcp->seq),
-	     ntohl(packet->tcp->seq)+packet->payload_packet_len);
+	     seq,
+	     seq + payload_len);
 #endif
 
-      message->next_seq = ntohl(packet->tcp->seq)+packet->payload_packet_len;
+      message->next_seq = seq + payload_len;
     } else {
 #ifdef DEBUG_TLS_MEMORY
-      printf("[TLS Mem] Skipping packet [%u bytes][direction: %u][tcp_seq: %u][expected next: %u]\n",
+      printf("[TLS Mem] Skipping packet [%u bytes][tcp_seq: %u][expected next: %u]\n",
 	     message->buffer_len,
-	     packet->packet_direction,
-	     ntohl(packet->tcp->seq),
-	     ntohl(packet->tcp->seq)+packet->payload_packet_len);
+	     seq,
+	     message->next_seq);
 #endif
     }
   }
+  return 0;
 }
 
 /* **************************************** */
@@ -292,6 +289,42 @@ static int extractRDNSequence(struct ndpi_packet_struct *packet,
 
 /* **************************************** */
 
+static u_int32_t make_tls_cert_key(struct ndpi_packet_struct *packet, int is_from_client)
+{
+  u_int32_t key;
+
+  /* Server ip/port */
+  if(packet->iphv6 == NULL) {
+    if(packet->tcp) {
+      if(is_from_client)
+        key = packet->iph->daddr + packet->tcp->dest;
+      else
+        key = packet->iph->saddr + packet->tcp->source;
+    } else {
+      if(is_from_client)
+        key = packet->iph->daddr + packet->udp->dest;
+      else
+        key = packet->iph->saddr + packet->udp->source;
+    }
+  } else {
+    if(packet->tcp) {
+      if(is_from_client)
+        key = ndpi_quick_hash((unsigned char *)&packet->iphv6->ip6_dst, 16) + packet->tcp->dest;
+      else
+        key = ndpi_quick_hash((unsigned char *)&packet->iphv6->ip6_src, 16) + packet->tcp->source;
+    } else {
+      if(is_from_client)
+        key = ndpi_quick_hash((unsigned char *)&packet->iphv6->ip6_dst, 16) + packet->udp->dest;
+      else
+        key = ndpi_quick_hash((unsigned char *)&packet->iphv6->ip6_src, 16) + packet->udp->source;
+    }
+  }
+
+  return key;
+}
+
+/* **************************************** */
+
 static void checkTLSSubprotocol(struct ndpi_detection_module_struct *ndpi_struct,
 				struct ndpi_flow_struct *flow,
 				int is_from_client) {
@@ -300,18 +333,16 @@ static void checkTLSSubprotocol(struct ndpi_detection_module_struct *ndpi_struct
   if(flow->detected_protocol_stack[1] == NDPI_PROTOCOL_UNKNOWN) {
     /* Subprotocol not yet set */
 
-    if(ndpi_struct->tls_cert_cache && packet->iph && packet->tcp) {
-      u_int32_t key; /* Server ip/port */
+    if(ndpi_struct->tls_cert_cache) {
       u_int16_t cached_proto;
+      u_int32_t key;
 
-      if(is_from_client)
-        key = packet->iph->daddr + packet->tcp->dest;
-      else
-        key = packet->iph->saddr + packet->tcp->source;
+      key = make_tls_cert_key(packet, is_from_client);
 
       if(ndpi_lru_find_cache(ndpi_struct->tls_cert_cache, key,
-			     &cached_proto, 0 /* Don't remove it as it can be used for other connections */)) {
-	ndpi_protocol ret = { __get_master(ndpi_struct, flow), cached_proto, NDPI_PROTOCOL_CATEGORY_UNSPECIFIED, NULL};
+			     &cached_proto, 0 /* Don't remove it as it can be used for other connections */,
+			     ndpi_get_current_time(flow))) {
+	ndpi_protocol ret = { __get_master(ndpi_struct, flow), cached_proto, NDPI_PROTOCOL_UNKNOWN /* unused */, NDPI_PROTOCOL_CATEGORY_UNSPECIFIED, NULL};
 
 	ndpi_set_detected_protocol(ndpi_struct, flow, cached_proto, __get_master(ndpi_struct, flow), NDPI_CONFIDENCE_DPI_CACHE);
 	flow->category = ndpi_get_proto_category(ndpi_struct, ret);
@@ -693,19 +724,16 @@ static void processCertificateElements(struct ndpi_detection_module_struct *ndpi
       if(rc == 0) {
 	/* Match found */
 	u_int16_t proto_id = (u_int16_t)val;
-	ndpi_protocol ret = { __get_master(ndpi_struct, flow), proto_id, NDPI_PROTOCOL_CATEGORY_UNSPECIFIED, NULL};
+	ndpi_protocol ret = { __get_master(ndpi_struct, flow), proto_id, NDPI_PROTOCOL_UNKNOWN /* unused */, NDPI_PROTOCOL_CATEGORY_UNSPECIFIED, NULL};
 
 	ndpi_set_detected_protocol(ndpi_struct, flow, proto_id, __get_master(ndpi_struct, flow), NDPI_CONFIDENCE_DPI);
 	flow->category = ndpi_get_proto_category(ndpi_struct, ret);
 	ndpi_check_subprotocol_risk(ndpi_struct, flow, proto_id);
 
-	if(ndpi_struct->tls_cert_cache == NULL)
-	  ndpi_struct->tls_cert_cache = ndpi_lru_cache_init(1024);
+	if(ndpi_struct->tls_cert_cache) {
+	  u_int32_t key = make_tls_cert_key(packet, 0 /* from the server */);
 
-	if(ndpi_struct->tls_cert_cache && packet->iph && packet->tcp) {
-	  u_int32_t key = packet->iph->saddr + packet->tcp->source; /* Server */
-
-	  ndpi_lru_add_to_cache(ndpi_struct->tls_cert_cache, key, proto_id);
+	  ndpi_lru_add_to_cache(ndpi_struct->tls_cert_cache, key, proto_id, ndpi_get_current_time(flow));
 	}
       }
     }
@@ -808,7 +836,7 @@ int processCertificate(struct ndpi_detection_module_struct *ndpi_struct,
 
       SHA1Final(flow->protos.tls_quic.sha1_certificate_fingerprint, &srv_cert_fingerprint_ctx);
 
-      flow->l4.tcp.tls.fingerprint_set = 1;
+      flow->protos.tls_quic.fingerprint_set = 1;
 
       uint8_t * sha1 = flow->protos.tls_quic.sha1_certificate_fingerprint;
       const size_t sha1_siz = sizeof(flow->protos.tls_quic.sha1_certificate_fingerprint);
@@ -854,8 +882,8 @@ int processCertificate(struct ndpi_detection_module_struct *ndpi_struct,
 
 /* **************************************** */
 
-int processTLSBlock(struct ndpi_detection_module_struct *ndpi_struct,
-                    struct ndpi_flow_struct *flow) {
+static int processTLSBlock(struct ndpi_detection_module_struct *ndpi_struct,
+                           struct ndpi_flow_struct *flow) {
   struct ndpi_packet_struct *packet = &ndpi_struct->packet;
   int ret;
 
@@ -868,6 +896,7 @@ int processTLSBlock(struct ndpi_detection_module_struct *ndpi_struct,
   case 0x02: /* Server Hello */
     processClientServerHello(ndpi_struct, flow, 0);
     flow->protos.tls_quic.hello_processed = 1;
+    flow->protos.tls_quic.ch_direction = (packet->payload[0] == 0x01 ? packet->packet_direction : !packet->packet_direction);
     ndpi_int_tls_add_connection(ndpi_struct, flow);
 
 #ifdef DEBUG_TLS
@@ -876,9 +905,10 @@ int processTLSBlock(struct ndpi_detection_module_struct *ndpi_struct,
 	   (packet->payload[0] == 0x01) ? "Client" : "Server");
 #endif
 
-    if((flow->protos.tls_quic.ssl_version >= 0x0304 /* TLS 1.3 */)
+    /* Not support for DTLS 1.3 yet, then certificates are always visible in DTLS */
+    if((packet->tcp && flow->protos.tls_quic.ssl_version >= 0x0304 /* TLS 1.3 */)
        && (packet->payload[0] == 0x02 /* Server Hello */)) {
-      flow->l4.tcp.tls.certificate_processed = 1; /* No Certificate with TLS 1.3+ */
+      flow->tls_quic.certificate_processed = 1; /* No Certificate with TLS 1.3+ */
     }
 
     checkTLSSubprotocol(ndpi_struct, flow, packet->payload[0] == 0x01);
@@ -888,13 +918,20 @@ int processTLSBlock(struct ndpi_detection_module_struct *ndpi_struct,
     /* Important: populate the tls union fields only after
      * ndpi_int_tls_add_connection has been called */
     if(flow->protos.tls_quic.hello_processed) {
-      ret = processCertificate(ndpi_struct, flow);
-      if(ret != 1) {
+      /* Only certificates from the server */
+      if(flow->protos.tls_quic.ch_direction != packet->packet_direction) {
+        ret = processCertificate(ndpi_struct, flow);
+        if(ret != 1) {
 #ifdef DEBUG_TLS
-        printf("[TLS] Error processing certificate: %d\n", ret);
+          printf("[TLS] Error processing certificate: %d\n", ret);
+#endif
+        }
+      } else {
+#ifdef DEBUG_TLS
+        printf("[TLS] Certificate from client. Ignoring it\n");
 #endif
       }
-      flow->l4.tcp.tls.certificate_processed = 1;
+      flow->tls_quic.certificate_processed = 1;
     }
     break;
 
@@ -922,15 +959,27 @@ static int ndpi_search_tls_tcp(struct ndpi_detection_module_struct *ndpi_struct,
   message_t *message;
 
 #ifdef DEBUG_TLS_MEMORY
-  printf("[TLS Mem] ndpi_search_tls_tcp() Processing new packet [payload_packet_len: %u]\n",
-	 packet->payload_packet_len);
+  printf("[TLS Mem] ndpi_search_tls_tcp() Processing new packet [payload_packet_len: %u][Dir: %u]\n",
+	 packet->payload_packet_len, packet->packet_direction);
 #endif
 
-  if(packet->payload_packet_len == 0)
-    return(1); /* Keep working */
+  /* This function is also called by "extra dissection" data path. Unfortunately,
+     generic "extra function" code doesn't honour protocol bitmask.
+     TODO: handle that in ndpi_main.c for all the protocols */
+  if(packet->payload_packet_len == 0 ||
+     packet->tcp_retransmission) {
+#ifdef DEBUG_TLS_MEMORY
+    printf("[TLS Mem] Ack or retransmission %d/%d. Skip\n",
+           packet->payload_packet_len, packet->tcp_retransmission);
+#endif
+    return 1; /* Keep working */
+  }
 
-  ndpi_search_tls_tcp_memory(ndpi_struct, flow);
-  message = &flow->l4.tcp.tls.message[packet->packet_direction];
+  message = &flow->tls_quic.message[packet->packet_direction];
+  if(ndpi_search_tls_memory(ndpi_struct, flow, packet->payload,
+			    packet->payload_packet_len, ntohl(packet->tcp->seq),
+			    message) == -1)
+    return 0; /* Error -> stop */
 
   /* Valid TLS Content Types:
      https://www.iana.org/assignments/tls-parameters/tls-parameters.xhtml#tls-parameters-5 */
@@ -1011,7 +1060,7 @@ static int ndpi_search_tls_tcp(struct ndpi_detection_module_struct *ndpi_struct,
 
     if((len > 9)
        && (content_type != 0x17 /* Application Data */)
-       && (!flow->l4.tcp.tls.certificate_processed)) {
+       && (!flow->tls_quic.certificate_processed)) {
       /* Split the element in blocks */
       u_int16_t processed = 5;
 
@@ -1058,9 +1107,9 @@ static int ndpi_search_tls_tcp(struct ndpi_detection_module_struct *ndpi_struct,
 	   we are after the handshake. Stop extra processing */
 	flow->l4.tcp.tls.app_data_seen[packet->packet_direction] = 1;
 	if(flow->l4.tcp.tls.app_data_seen[!packet->packet_direction] == 1)
-	  flow->l4.tcp.tls.certificate_processed = 1;
+	  flow->tls_quic.certificate_processed = 1;
 
-	if(flow->l4.tcp.tls.certificate_processed) {
+	if(flow->tls_quic.certificate_processed) {
 	  if(flow->l4.tcp.tls.num_tls_blocks < ndpi_struct->num_tls_blocks_to_follow)
 	    flow->l4.tcp.tls.tls_application_blocks_len[flow->l4.tcp.tls.num_tls_blocks++] =
 	      (packet->packet_direction == 0) ? (len-5) : -(len-5);
@@ -1091,12 +1140,11 @@ static int ndpi_search_tls_tcp(struct ndpi_detection_module_struct *ndpi_struct,
      || ((ndpi_struct->num_tls_blocks_to_follow > 0)
 	 && (flow->l4.tcp.tls.num_tls_blocks == ndpi_struct->num_tls_blocks_to_follow))
      || ((ndpi_struct->num_tls_blocks_to_follow == 0)
-	 && (flow->l4.tcp.tls.certificate_processed == 1))
+	 && (flow->tls_quic.certificate_processed == 1))
      ) {
 #ifdef DEBUG_TLS_BLOCKS
     printf("*** [TLS Block] No more blocks\n");
 #endif
-    flow->check_extra_packets = 0;
     flow->extra_packets_func = NULL;
     return(0); /* That's all */
   } else
@@ -1108,10 +1156,11 @@ static int ndpi_search_tls_tcp(struct ndpi_detection_module_struct *ndpi_struct,
 static int ndpi_search_tls_udp(struct ndpi_detection_module_struct *ndpi_struct,
 			       struct ndpi_flow_struct *flow) {
   struct ndpi_packet_struct *packet = &ndpi_struct->packet;
-  u_int32_t handshake_len;
+  u_int32_t handshake_len, handshake_frag_off, handshake_frag_len;
   u_int16_t p_len, processed;
   const u_int8_t *p;
   u_int8_t no_dtls = 0, change_cipher_found = 0;
+  message_t *message = NULL;
 
 #ifdef DEBUG_TLS
   printf("[TLS] %s()\n", __FUNCTION__);
@@ -1159,19 +1208,73 @@ static int ndpi_search_tls_udp(struct ndpi_detection_module_struct *ndpi_struct,
         break;
       }
       /* TODO: handle (certificate) fragments */
-      if(block_len > 16) {
+      if(block_len > 24) {
         handshake_len = (block[14] << 16) + (block[15] << 8) + block[16];
-        if((handshake_len + 12) != block_len) {
+        handshake_frag_off = (block[19] << 16) + (block[20] << 8) + block[21];
+        handshake_frag_len = (block[22] << 16) + (block[23] << 8) + block[24];
+        message = &flow->tls_quic.message[packet->packet_direction];
+
+
 #ifdef DEBUG_TLS
-          printf("[TLS] DTLS invalid handshake_len %d, %d)\n",
+        printf("[TLS] DTLS frag off %d len %d\n", handshake_frag_off, handshake_frag_len);
+#endif
+
+	if((handshake_len + 12) == block_len) {
+          packet->payload = &block[13];
+          packet->payload_packet_len = block_len;
+          processTLSBlock(ndpi_struct, flow);
+	} else if(handshake_len + 12 > block_len) {
+	  int rc;
+
+#ifdef DEBUG_TLS
+          printf("[TLS] DTLS fragment off %d len %d\n", handshake_frag_off, handshake_frag_len);
+#endif
+          if(handshake_frag_len + 12 > block_len) {
+#ifdef DEBUG_TLS
+            printf("[TLS] DTLS fragment invalid len %d + 12 > %d\n", handshake_frag_len, block_len);
+#endif
+            no_dtls = 1;
+            break;
+	  }
+
+          if(handshake_frag_off == 0) {
+            rc = ndpi_search_tls_memory(ndpi_struct, flow,  &block[13],
+					handshake_frag_len + 12,
+					handshake_frag_off, message);
+	  } else {
+            rc = ndpi_search_tls_memory(ndpi_struct, flow,  &block[13 + 12],
+					handshake_frag_len,
+					handshake_frag_off + 12, message);
+	  }
+	  if(rc == -1) {
+            no_dtls = 1;
+            break;
+	  }
+#ifdef DEBUG_TLS
+          printf("[TLS] DTLS reassembled len %d vs %d\n",
+                 message->buffer_used, handshake_len + 12);
+#endif
+
+          if(handshake_len + 12 == message->buffer_used) {
+            packet->payload = message->buffer;
+            packet->payload_packet_len = message->buffer_used;
+            processTLSBlock(ndpi_struct, flow);
+
+            ndpi_free(message->buffer);
+            memset(message, '\0', sizeof(*message));
+            message = NULL;
+          } else {
+            /* No break, next fragments might be in the same packet */
+          }
+
+        } else {
+#ifdef DEBUG_TLS
+          printf("[TLS] DTLS invalid handshake_len %d, %d\n",
                  handshake_len, block_len);
 #endif
           no_dtls = 1;
           break;
         }
-        packet->payload = &block[13];
-        packet->payload_packet_len = block_len;
-        processTLSBlock(ndpi_struct, flow);
       }
     } else if(block[0] == 0x14) {
       /* Change-cipher-spec: any subsequent block might be encrypted */
@@ -1188,13 +1291,13 @@ static int ndpi_search_tls_udp(struct ndpi_detection_module_struct *ndpi_struct,
       processed += block_len + 13;
       /* DTLS mid session: no need to further inspect the flow */
       ndpi_set_detected_protocol(ndpi_struct, flow, NDPI_PROTOCOL_DTLS, NDPI_PROTOCOL_UNKNOWN, NDPI_CONFIDENCE_DPI);
-      flow->l4.tcp.tls.certificate_processed = 1; /* Fake, to avoid extra dissection */
+      flow->tls_quic.certificate_processed = 1; /* Fake, to avoid extra dissection */
       break;
     }
 
     processed += block_len + 13;
   }
-  if(processed != p_len) {
+  if(processed != p_len && message == NULL /* No pending reassembler */) {
 #ifdef DEBUG_TLS
     printf("[TLS] DTLS invalid processed len %d/%d (%d)\n", processed, p_len, change_cipher_found);
 #endif
@@ -1205,9 +1308,8 @@ static int ndpi_search_tls_udp(struct ndpi_detection_module_struct *ndpi_struct,
   packet->payload = p;
   packet->payload_packet_len = p_len; /* Restore */
 
-  if(no_dtls || change_cipher_found || flow->l4.tcp.tls.certificate_processed) {
+  if(no_dtls || change_cipher_found || flow->tls_quic.certificate_processed) {
     NDPI_EXCLUDE_PROTO_EXT(ndpi_struct, flow, NDPI_PROTOCOL_DTLS);
-    flow->check_extra_packets = 0;
     flow->extra_packets_func = NULL;
     return(0); /* That's all */
   } else {
@@ -1221,8 +1323,6 @@ static void tlsInitExtraPacketProcessing(struct ndpi_detection_module_struct *nd
 					 struct ndpi_flow_struct *flow) {
   struct ndpi_packet_struct *packet = &ndpi_struct->packet;
 
-  flow->check_extra_packets = 1;
-
   /* At most 12 packets should almost always be enough to find the server certificate if it's there */
   flow->max_extra_packets_to_check = 12 + (ndpi_struct->num_tls_blocks_to_follow*4);
   flow->extra_packets_func = (packet->udp != NULL) ? ndpi_search_tls_udp : ndpi_search_tls_tcp;
@@ -1230,10 +1330,51 @@ static void tlsInitExtraPacketProcessing(struct ndpi_detection_module_struct *nd
 
 /* **************************************** */
 
+void switch_extra_dissection_to_tls(struct ndpi_detection_module_struct *ndpi_struct,
+				    struct ndpi_flow_struct *flow)
+{
+#ifdef DEBUG_TLS
+  printf("Switching to TLS extra dissection\n");
+#endif
+
+  /* Reset reassemblers */
+  if(flow->tls_quic.message[0].buffer)
+    ndpi_free(flow->tls_quic.message[0].buffer);
+  memset(&flow->tls_quic.message[0], '\0', sizeof(flow->tls_quic.message[0]));
+  if(flow->tls_quic.message[1].buffer)
+    ndpi_free(flow->tls_quic.message[1].buffer);
+  memset(&flow->tls_quic.message[1], '\0', sizeof(flow->tls_quic.message[1]));
+
+  tlsInitExtraPacketProcessing(ndpi_struct, flow);
+}
+
+/* **************************************** */
+
+static void tls_subclassify_by_alpn(struct ndpi_detection_module_struct *ndpi_struct,
+				    struct ndpi_flow_struct *flow) {
+  /* Right now we have only one rule so we can keep it trivial */
+
+  if (!flow->protos.tls_quic.advertised_alpns)
+    return;
+
+  if(strlen(flow->protos.tls_quic.advertised_alpns) > NDPI_STATICSTRING_LEN("anydesk/") &&
+     strncmp(flow->protos.tls_quic.advertised_alpns, "anydesk/", NDPI_STATICSTRING_LEN("anydesk/")) == 0) {
+#ifdef DEBUG_TLS
+    printf("Matching ANYDESK via alpn\n");
+#endif
+    ndpi_set_detected_protocol(ndpi_struct, flow, NDPI_PROTOCOL_ANYDESK,
+			       __get_master(ndpi_struct, flow), NDPI_CONFIDENCE_DPI);
+    flow->protos.tls_quic.subprotocol_detected = 1;
+  }
+}
+
+/* **************************************** */
+
 static void tlsCheckUncommonALPN(struct ndpi_detection_module_struct *ndpi_struct,
-				 struct ndpi_flow_struct *flow) {
-  char * alpn_start = flow->protos.tls_quic.alpn_server;
+				 struct ndpi_flow_struct *flow,
+				 char *alpn_start) {
   char * comma_or_nul = alpn_start;
+
   do {
     size_t alpn_len;
 
@@ -1279,7 +1420,7 @@ static void ndpi_int_tls_add_connection(struct ndpi_detection_module_struct *ndp
 
   if((flow->detected_protocol_stack[0] != NDPI_PROTOCOL_UNKNOWN) ||
      (flow->detected_protocol_stack[1] != NDPI_PROTOCOL_UNKNOWN)) {
-    if(!flow->check_extra_packets)
+    if(!flow->extra_packets_func)
       tlsInitExtraPacketProcessing(ndpi_struct, flow);
     return;
   }
@@ -1508,7 +1649,7 @@ int processClientServerHello(struct ndpi_detection_module_struct *ndpi_struct,
 	  u_int16_t s_offset = offset+4;
 	  u_int16_t tot_alpn_len = ntohs(*((u_int16_t*)&packet->payload[s_offset]));
 	  char alpn_str[256];
-	  u_int8_t alpn_str_len = 0, i;
+	  u_int16_t alpn_str_len = 0, i;
 
 #ifdef DEBUG_TLS
 	  printf("Server TLS [ALPN: block_len=%u/len=%u]\n", extension_len, tot_alpn_len);
@@ -1558,13 +1699,18 @@ int processClientServerHello(struct ndpi_detection_module_struct *ndpi_struct,
 	  if(ndpi_normalize_printable_string(alpn_str, alpn_str_len) == 0)
 	    ndpi_set_risk(ndpi_struct, flow, NDPI_INVALID_CHARACTERS, alpn_str);
 
-	  if(flow->protos.tls_quic.alpn_server == NULL)
-	    flow->protos.tls_quic.alpn_server = ndpi_strdup(alpn_str);
+	  if(flow->protos.tls_quic.negotiated_alpn == NULL)
+	    flow->protos.tls_quic.negotiated_alpn = ndpi_strdup(alpn_str);
 
-	  if(flow->protos.tls_quic.alpn_server != NULL)
-	    tlsCheckUncommonALPN(ndpi_struct, flow);
+	  /* Check ALPN only if not already checked (client-side) */
+	  if(flow->protos.tls_quic.negotiated_alpn != NULL &&
+	     flow->protos.tls_quic.advertised_alpns == NULL)
+	    tlsCheckUncommonALPN(ndpi_struct, flow, flow->protos.tls_quic.negotiated_alpn);
 
-	  ndpi_snprintf(ja3.server.alpn, sizeof(ja3.server.alpn), "%s", alpn_str);
+	  alpn_str_len = ndpi_min(sizeof(ja3.server.alpn), (size_t)alpn_str_len);
+	  memcpy(ja3.server.alpn, alpn_str, alpn_str_len);
+	  if(alpn_str_len > 0)
+	    ja3.server.alpn[alpn_str_len - 1] = '\0';
 
 	  /* Replace , with - as in JA3 */
 	  for(i=0; ja3.server.alpn[i] != '\0'; i++)
@@ -1603,6 +1749,11 @@ int processClientServerHello(struct ndpi_detection_module_struct *ndpi_struct,
 	i += 4 + extension_len, offset += 4 + extension_len;
       } /* for */
 
+      /* If the CH is not available and if "supported_versions" extension is not present in the SH
+         (i.e. (D)TLS <= 1.2), use the version field present in the record layer */
+      if(flow->protos.tls_quic.ssl_version == 0)
+        flow->protos.tls_quic.ssl_version = tls_version;
+
       ja3_str_len = ndpi_snprintf(ja3_str, JA3_STR_LEN, "%u,", ja3.server.tls_handshake_version);
 
       for(i=0; (i<ja3.server.num_cipher) && (JA3_STR_LEN > ja3_str_len); i++) {
@@ -1902,7 +2053,7 @@ int processClientServerHello(struct ndpi_detection_module_struct *ndpi_struct,
 		    }
 
 		    if(ndpi_check_dga_name(ndpi_struct, flow,
-					   sni, 1)) {
+					   sni, 1, 0)) {
 #ifdef DEBUG_TLS
 		      printf("[TLS] SNI: (DGA) [%s]\n", sni);
 #endif
@@ -2117,7 +2268,7 @@ int processClientServerHello(struct ndpi_detection_module_struct *ndpi_struct,
 		u_int16_t s_offset = offset+extension_offset;
 		u_int16_t tot_alpn_len = ntohs(*((u_int16_t*)&packet->payload[s_offset]));
 		char alpn_str[256];
-		u_int8_t alpn_str_len = 0, i;
+		u_int16_t alpn_str_len = 0, i;
 
 #ifdef DEBUG_TLS
 		printf("Client TLS [ALPN: block_len=%u/len=%u]\n", extension_len, tot_alpn_len);
@@ -2155,10 +2306,23 @@ int processClientServerHello(struct ndpi_detection_module_struct *ndpi_struct,
 #ifdef DEBUG_TLS
 		printf("Client TLS [ALPN: %s][len: %u]\n", alpn_str, alpn_str_len);
 #endif
-		if(flow->protos.tls_quic.alpn == NULL)
-		  flow->protos.tls_quic.alpn = ndpi_strdup(alpn_str);
+		if(flow->protos.tls_quic.advertised_alpns == NULL) {
+		  flow->protos.tls_quic.advertised_alpns = ndpi_strdup(alpn_str);
+		  if(flow->protos.tls_quic.advertised_alpns) {
+		    tlsCheckUncommonALPN(ndpi_struct, flow, flow->protos.tls_quic.advertised_alpns);
 
-		ndpi_snprintf(ja3.client.alpn, sizeof(ja3.client.alpn), "%s", alpn_str);
+		    /* Without SNI matching we can try to sub-classify the flow via ALPN.
+		       Note that this happens only on very rare cases, not the common ones
+		       ("h2", "http/1.1", ...). Usefull for asymmetric traffic */
+		    if(!flow->protos.tls_quic.subprotocol_detected)
+	              tls_subclassify_by_alpn(ndpi_struct, flow);
+		  }
+		}
+
+                alpn_str_len = ndpi_min(sizeof(ja3.client.alpn), (size_t)alpn_str_len);
+		memcpy(ja3.client.alpn, alpn_str, alpn_str_len);
+		if(alpn_str_len > 0)
+		  ja3.client.alpn[alpn_str_len - 1] = '\0';
 
 		/* Replace , with - as in JA3 */
 		for(i=0; ja3.client.alpn[i] != '\0'; i++)
@@ -2419,7 +2583,7 @@ int processClientServerHello(struct ndpi_detection_module_struct *ndpi_struct,
 
 	    /* Before returning to the caller we need to make a final check */
 	    if((flow->protos.tls_quic.ssl_version >= 0x0303) /* >= TLSv1.2 */
-	       && (flow->protos.tls_quic.alpn == NULL) /* No ALPN */) {
+	       && (flow->protos.tls_quic.advertised_alpns == NULL) /* No ALPN */) {
 	      ndpi_set_risk(ndpi_struct, flow, NDPI_TLS_NOT_CARRYING_HTTPS, "No ALPN");
 	    }
 
@@ -2470,7 +2634,7 @@ static void ndpi_search_tls_wrapper(struct ndpi_detection_module_struct *ndpi_st
 #ifdef DEBUG_TLS
   printf("==>> %s() %u [len: %u][version: %u]\n",
 	 __FUNCTION__,
-	 flow->guessed_host_protocol_id,
+	 flow->guessed_protocol_id_by_ip,
 	 packet->payload_packet_len,
 	 flow->protos.tls_quic.ssl_version);
 #endif
@@ -2484,8 +2648,8 @@ static void ndpi_search_tls_wrapper(struct ndpi_detection_module_struct *ndpi_st
 /* **************************************** */
 
 void init_tls_dissector(struct ndpi_detection_module_struct *ndpi_struct,
-			u_int32_t *id, NDPI_PROTOCOL_BITMASK *detection_bitmask) {
-  ndpi_set_bitmask_protocol_detection("TLS", ndpi_struct, detection_bitmask, *id,
+			u_int32_t *id) {
+  ndpi_set_bitmask_protocol_detection("TLS", ndpi_struct, *id,
 				      NDPI_PROTOCOL_TLS,
 				      ndpi_search_tls_wrapper,
 				      NDPI_SELECTION_BITMASK_PROTOCOL_V4_V6_TCP_WITH_PAYLOAD_WITHOUT_RETRANSMISSION,
@@ -2496,7 +2660,7 @@ void init_tls_dissector(struct ndpi_detection_module_struct *ndpi_struct,
 
   /* *************************************************** */
 
-  ndpi_set_bitmask_protocol_detection("DTLS", ndpi_struct, detection_bitmask, *id,
+  ndpi_set_bitmask_protocol_detection("DTLS", ndpi_struct, *id,
 				      NDPI_PROTOCOL_DTLS,
 				      ndpi_search_tls_wrapper,
 				      NDPI_SELECTION_BITMASK_PROTOCOL_V4_V6_UDP_WITH_PAYLOAD,
