@@ -85,6 +85,9 @@ using namespace std;
 
 #include "nd-config.h"
 #include "nd-ndpi.h"
+#include "nd-risks.h"
+#include "nd-serializer.h"
+#include "nd-packet.h"
 #include "nd-sha1.h"
 #include "nd-json.h"
 #include "nd-util.h"
@@ -216,6 +219,26 @@ void nd_ndpi_debug_printf(uint32_t protocol, void *ndpi,
 }
 #endif // NDPI_ENABLE_DEBUG_MESSAGES
 
+int ndLogBuffer::overflow(int ch)
+{
+    if (ch != EOF)
+        os << (char)ch;
+
+    if (ch == '\n') return sync();
+
+    return 0;
+}
+
+int ndLogBuffer::sync()
+{
+    if (! os.str().empty()) {
+        nd_printf("%s", os.str().c_str());
+        os.str("");
+    }
+
+    return 0;
+}
+
 void nd_print_address(const struct sockaddr_storage *addr)
 {
     int rc;
@@ -258,7 +281,6 @@ void nd_print_number(ostringstream &os, uint64_t value, bool units_binary)
     float fvalue = value;
 
     os.str("");
-    //os << setiosflags(ios::fixed) << setw(14) << setprecision(3);
     os << setw(8) << setprecision(3);
 
     if (units_binary) {
@@ -305,6 +327,13 @@ void nd_print_number(ostringstream &os, uint64_t value, bool units_binary)
     }
 }
 
+void nd_print_percent(ostringstream &os, const double &value)
+{
+    os.str("");
+    os << setw(8) << setprecision(3);
+    os << value << " " << "%";
+}
+
 void nd_ltrim(string &s, unsigned char c)
 {
     s.erase(s.begin(), find_if(s.begin(), s.end(), [c](unsigned char ch) {
@@ -874,116 +903,6 @@ int nd_ifreq(const string &name, unsigned long request, struct ifreq *ifr)
     return rc;
 }
 
-int nd_ifaddrs(nd_interface_addr_map &addr_map)
-{
-    int count = 0;
-    struct ifaddrs *ifaddr, *ifa;
-    nd_interface_addr_map::iterator i;
-    nd_interface_addr_array *addrs;
-
-    if (getifaddrs(&ifaddr) == -1) {
-        nd_printf("getifaddrs: %s\n", strerror(errno));
-        return -1;
-    }
-
-    for (ifa = ifaddr; ifa != NULL; ifa = ifa->ifa_next) {
-        if (ifa->ifa_addr == NULL) continue;
-
-        i = addr_map.find(ifa->ifa_name);
-        if (i == addr_map.end()) {
-            nd_interface_addr_insert addr_iter;
-
-            addrs = new nd_interface_addr_array;
-            if (addrs == NULL)
-                throw runtime_error(strerror(ENOMEM));
-            addr_iter = addr_map.insert(nd_interface_addr_pair(ifa->ifa_name, addrs));
-            i = addr_iter.first;
-        }
-        else
-            addrs = i->second;
-
-        struct ndInterfaceAddress *addr = new struct ndInterfaceAddress;
-        if (addr == NULL) throw runtime_error(strerror(ENOMEM));
-        memset(addr, 0, sizeof(struct ndInterfaceAddress));
-
-        addr->family = ifa->ifa_addr->sa_family;
-
-        switch (addr->family) {
-        case AF_LINK:
-            memset(addr->mac, 0, ETH_ALEN);
-#if defined(__linux__)
-            {
-                struct sockaddr_ll *s = (struct sockaddr_ll*)ifa->ifa_addr;
-                memcpy(addr->mac, s->sll_addr, ETH_ALEN);
-            }
-#elif defined(BSD4_4)
-            {
-                struct sockaddr_dl *s = (struct sockaddr_dl *)ifa->ifa_addr;
-                memcpy(addr->mac, s->sdl_data + s->sdl_nlen, ETH_ALEN);
-            }
-#endif
-            addrs->push_back(addr);
-            count++;
-            break;
-        case AF_INET:
-            memcpy(&addr->ip, ifa->ifa_addr, sizeof(struct sockaddr_in));
-            addrs->push_back(addr);
-            count++;
-            break;
-        case AF_INET6:
-            memcpy(&addr->ip, ifa->ifa_addr, sizeof(struct sockaddr_in6));
-            addrs->push_back(addr);
-            count++;
-            break;
-        default:
-            delete addr;
-        }
-    }
-
-    freeifaddrs(ifaddr);
-
-    return count;
-}
-
-int nd_ifaddrs_update(nd_interface_addr_map &addr_map)
-{
-    nd_ifaddrs_free(addr_map);
-    return nd_ifaddrs(addr_map);
-}
-
-void nd_ifaddrs_free(nd_interface_addr_map &addr_map)
-{
-    nd_interface_addr_map::iterator i;
-
-    for (i = addr_map.begin(); i != addr_map.end(); i++) {
-        vector<struct ndInterfaceAddress *>::iterator a;
-        for (a = i->second->begin(); a != i->second->end(); a++)
-            delete (*a);
-        delete (i->second);
-    }
-
-    addr_map.clear();
-}
-
-bool nd_ifaddrs_get_mac(
-    nd_interface_addr_map &addr_map,
-    const string &name, uint8_t *addr)
-{
-    nd_interface_addr_map::iterator i = addr_map.find(name);
-
-    if (i == addr_map.end()) return false;
-
-    vector<struct ndInterfaceAddress *>::iterator a;
-    for (a = i->second->begin(); a != i->second->end(); a++) {
-        if ((*a)->family == AF_LINK) {
-            memcpy(addr, (*a)->mac, ETH_ALEN);
-            return true;
-        }
-    }
-
-    return false;
-}
-
 #if defined(__linux__)
 pid_t nd_is_running(pid_t pid, const char *exe_base)
 {
@@ -1302,35 +1221,30 @@ bool nd_scan_dotd(const string &path, vector<string> &files)
         return false;
     }
 
-    long dname_max = pathconf(path.c_str(), _PC_NAME_MAX);
-    if (dname_max == -1) dname_max = 255;
-    size_t entry_size = offsetof(struct dirent, d_name) + dname_max + 1;
+    files.clear();
 
-    struct dirent *dentry = reinterpret_cast<struct dirent *>(
-        new uint8_t[entry_size]
-    );
-    unique_ptr<struct dirent, void(*)(struct dirent *p)> entry(
-        dentry, [](struct dirent *p) {
-        delete [] (uint8_t *)p;
-    });
-
-    int rc;
     struct dirent *result = NULL;
-    while (! (rc = readdir_r(dh, dentry, &result))) {
-        if (result == NULL) break;
+    while ((result = readdir(dh)) != NULL) {
         if (
+#ifdef _DIRENT_HAVE_D_RECLEN
+            result->d_reclen == 0 ||
+#endif
+#ifndef _DIRENT_HAVE_D_TYPE
+#warning "struct dirent doesn't have a type!"
+#else
             (
                 result->d_type != DT_LNK &&
                 result->d_type != DT_REG &&
                 result->d_type != DT_UNKNOWN
             )
+#endif
             || ! isdigit(result->d_name[0])) continue;
         files.push_back(result->d_name);
     }
 
     closedir(dh);
 
-    return (rc == 0);
+    return (files.size () > 0);
 }
 
 void nd_set_hostname(char *dst,
